#ifndef CONSTANT_CPP
#define CONSTANT_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Constant
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple Constant object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Constant.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Constant
//
// DESCRIPTION: Constructor for Constant objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Constant::Constant(Coordinate* pCoord,
		   const int nCoord,
		   const DisplayMode eDisplayMode,
		   const word wValue)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_pwValue = new word[1];
  m_pwValue[0] = wValue;
  m_eIsUpToDate = true; // Never update
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~Constant
//
// DESCRIPTION: Destructor for Constant objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Constant::~Constant(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: NewCycle
//
// DESCRIPTION: This public method is called each time a new cycle begins.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Constant::NewCycle(void)
{
  m_eIsActivated = false; // The component hasn't been activated yet.
  m_eIsUpToDate = true;   // The component must NEVER be updated.
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Constant::GetValue(const int nIndex,const AskMode eAskMode)
{
  switch(nIndex)
    {
    case 0:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      // No need to update : this is a CONSTANT !
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Constant::Display(void)
{
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      


      break;
    case DISPLAY_TEXT:



      break;
    }
}

#endif // CONSTANT_CPP
