#ifndef DATAPATHCOMPONENT_HPP
#define DATAPATHCOMPONENT_HPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: DataPathComponent
//
// BASE CLASSES: None.
//
// PURPOSE: Generic class describing datapath components.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "../Core/Core.hpp"
#include "../Interface.hpp" // For graphic mode
#include <iostream>         // For console mode
#include <iomanip>          // Idem

//Constants

//Types and classes

typedef enum { ASK_PASV , ASK_ACTV } AskMode;
typedef enum { MOUSE_CLK } MouseEvent;
typedef enum { DISPLAY_GRAPHIC , DISPLAY_TEXT } DisplayMode;

typedef struct { int X; int Y; } Coordinate;

class DataPathComponent
{
public:
  DataPathComponent(Coordinate*,
		    const int,
		    const DisplayMode);
  virtual ~DataPathComponent(void);
  void DependsOn(int);
  void DependsOn(DataPathComponent*,int);
  virtual void NewCycle(void);
  virtual word GetValue(const int,
			const AskMode)=0;
  virtual void Display(void)=0;
  virtual bool MouseIsOver(Coordinate*);
  virtual void MouseAction(const MouseEvent);
protected:
  DataPathComponent** m_ppInput;
  int* m_pnOutput;
  int m_nInput;
  Coordinate* m_pCoord;
  int m_nCoord;
  DisplayMode m_eDisplayMode;
  bool m_eIsUpToDate;
  bool m_eIsActivated;
  word* m_pwValue;
};

#endif // DATAPATHCOMPONENT_HPP
