#ifndef GATEAND_CPP
#define GATEAND_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: GateAND
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple AND gate.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "GateAND.hpp"
#include "XPM/GateAND_disabled.xpm"
#include "XPM/GateAND_enabled.xpm"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GateAND
//
// DESCRIPTION: Constructor for GateAND objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
GateAND::GateAND(Coordinate* pCoord,
		 const int nCoord,
		 const DisplayMode eDisplayMode)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_pwValue = new word[1];
  m_pwValue[0] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~GateAND
//
// DESCRIPTION: Destructor for GateAND objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
GateAND::~GateAND(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
GateAND::GetValue(const int nIndex,const AskMode eAskMode)
{
  switch(nIndex)
    {
    case 0:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      //      if(m_eIsUpToDate == false)
	{
	  m_pwValue[nIndex] = 0x00000000;
	  for(int nInput = 0 ; nInput < m_nInput ; nInput++)
	    m_pwValue[nIndex] &=
	      m_ppInput[nInput]->GetValue(m_pnOutput[nInput],eAskMode);
	  //	  m_eIsUpToDate = true;
	}
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
GateAND::Display(void)
{
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      if(m_eIsActivated == true)
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)GateAND_enabled);
	}
      else
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)GateAND_disabled);
	}
      gdk_draw_drawable(pPixmap,
			pDrawingArea->style->fg_gc[GTK_WIDGET_STATE(pDrawingArea)],
			pPixmapTmp,0,0,
			m_pCoord[0].X,
			m_pCoord[0].Y,
			m_pCoord[1].X - m_pCoord[0].X,
			m_pCoord[1].Y - m_pCoord[0].Y);
      gdk_pixmap_unref(pPixmapTmp);
      gtk_widget_queue_draw_area (pDrawingArea,
				  0,0,
				  pDrawingArea->allocation.width,
				  pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:

      break;
    }
}

#endif // GATEAND_CPP
