#ifndef DATAMEMORY_CPP
#define DATAMEMORY_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: DataMemory
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple DataMemory object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "DataMemory.hpp"
#include "../XPM/DataMemory_disabled.xpm"
#include "../XPM/DataMemory_enabled.xpm"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DataMemory
//
// DESCRIPTION: Constructor for DataMemory objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataMemory::DataMemory(Coordinate* pCoord,
		       const int nCoord,
		       const DisplayMode eDisplayMode)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_pwValue = new word[1];
  m_pwValue[0] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~DataMemory
//
// DESCRIPTION: Destructor for DataMemory objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataMemory::~DataMemory(void)
{
  Reset();
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Init
//
// DESCRIPTION: This public method initializes the memory allocation table
//
// PARAMETERS: Allocation Table.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataMemory::Init(AllocTable* pAllocTable)
{
  m_Memory.Init(pAllocTable);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
DataMemory::GetValue(const int nIndex,const AskMode eAskMode)
{
  switch(nIndex)
    {
    case 0:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      if(m_eIsUpToDate == false)
	{
	  word wAddress = m_ppInput[0]->GetValue(m_pnOutput[0],eAskMode);
	  word wContent = m_ppInput[1]->GetValue(m_pnOutput[1],eAskMode);
	  byte MemRead  = m_ppInput[2]->GetValue(m_pnOutput[2],eAskMode);
	  byte MemWrite = m_ppInput[3]->GetValue(m_pnOutput[3],eAskMode);
	  switch(MemRead)
	    {
	    case 1:
	      // We are allowed to read
	      m_pwValue[0] = m_Memory[wAddress];
	      break;
	    case 0:
	    default:
	      // We aren't allowed to read
	      // So we return 0x00000000
	      m_pwValue[0] = 0x00000000;
	      break;
	    }
	  switch(MemWrite)
	    {
	    case 1:
	      // We are allowed to write
	      m_Memory[wAddress] = wContent;
	      break;
	    case 0:
	    default:
	      // We aren't allowed to write
	      // So we don't do anything
	      break;
	    }
	  m_eIsUpToDate = true;
	}
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This method is used to reset the memory.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataMemory::Reset(void)
{
  m_Memory.Reset();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataMemory::Display(void)
{
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      if(m_eIsActivated == true)
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)DataMemory_enabled);
	}
      else
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)DataMemory_disabled);
	}
      gdk_draw_drawable(pPixmap,
			pDrawingArea->style->fg_gc[GTK_WIDGET_STATE(pDrawingArea)],
			pPixmapTmp,0,0,
			m_pCoord[0].X,
			m_pCoord[0].Y,
			m_pCoord[1].X - m_pCoord[0].X,
			m_pCoord[1].Y - m_pCoord[0].Y);
      gdk_pixmap_unref(pPixmapTmp);
      gtk_widget_queue_draw_area(pDrawingArea,
				 0,0,
				 pDrawingArea->allocation.width,
				 pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:

      using namespace std;
      word wAddress = m_ppInput[0]->GetValue(m_pnOutput[0],ASK_PASV);
      byte MemRead  = m_ppInput[2]->GetValue(m_pnOutput[2],ASK_PASV);
      byte MemWrite = m_ppInput[3]->GetValue(m_pnOutput[3],ASK_PASV);
      cout << hex << setfill('0') << internal << uppercase ;
      if(MemWrite)
	{
	  cout << "Word 0x" << setw(8) << m_Memory[wAddress];
	  cout << " written at memory address ";
	  cout << "0x" << setw(8) << wAddress << endl;
	}
      if(MemRead)
	{
  	  cout << "Word 0x" << setw(8) << m_Memory[wAddress];
	  cout << " read from memory address ";
	  cout << "0x" << setw(8) << wAddress << endl;
	}

      break;
    }  
}

#endif // DATAMEMORY_CPP
