#ifndef REGISTERBOX_CPP
#define REGISTERBOX_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: RegisterBox
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple RegisterBox object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "RegisterBox.hpp"
#include "XPM/RegisterBox_disabled.xpm"
#include "XPM/RegisterBox_enabled.xpm"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: RegisterBox
//
// DESCRIPTION: Constructor for RegisterBox objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
RegisterBox::RegisterBox(Coordinate* pCoord,
			 const int nCoord,
			 const DisplayMode eDisplayMode)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_pwValue = new word[2];
  m_pwRegister = new word[MAX_REGISTERS];
  Reset();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~RegisterBox
//
// DESCRIPTION: Destructor for RegisterBox objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
RegisterBox::~RegisterBox(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
RegisterBox::GetValue(const int nIndex,const AskMode eAskMode)
{
  word ReadReg1 = 0x00000000;
  word ReadReg2 = 0x00000000;
  switch(nIndex)
    {
    case 0:
    case 1:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      if(nIndex == 0) // Rs
	{
	  ReadReg1 = m_ppInput[0]->GetValue(m_pnOutput[0],eAskMode);
	  ReadReg2 = m_ppInput[1]->GetValue(m_pnOutput[1],ASK_PASV);
	}
      else if(nIndex == 1) // Rt
	{
	  ReadReg1 = m_ppInput[0]->GetValue(m_pnOutput[0],ASK_PASV);
	  ReadReg2 = m_ppInput[1]->GetValue(m_pnOutput[1],eAskMode);
	}
      m_pwValue[0]   = m_pwRegister[ReadReg1];
      m_pwValue[1]   = m_pwRegister[ReadReg2];
      if(m_eIsUpToDate == false)
	{
	  word RegWrite  = m_ppInput[4]->GetValue(m_pnOutput[4],eAskMode);
	  m_eIsUpToDate  = true; // Prevents loop
	  if(Core::Bit(RegWrite,0) == 0x01)
	    {
	      m_eIsActivated = true; // If we need to write, we are active !
	      word WriteReg  = m_ppInput[2]->GetValue(m_pnOutput[2],ASK_ACTV);
	      word WriteData = m_ppInput[3]->GetValue(m_pnOutput[3],ASK_ACTV);
	      if(m_eDisplayMode == DISPLAY_GRAPHIC) // Graphic Mode
		m_ppInput[4]->GetValue(m_pnOutput[4],ASK_ACTV); // Update
	      if(WriteReg != 0) // Register $0 cannot be written !
		m_pwRegister[WriteReg] = WriteData;
	    }
	  else
	    {
	      m_ppInput[2]->GetValue(0,ASK_PASV);
	      m_ppInput[3]->GetValue(0,ASK_PASV);
	    }
	}
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This public method is used to reset the object.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
RegisterBox::Reset(void)
{
  // We reset the outputs :
  m_pwValue[0] = 0x00000000;
  m_pwValue[1] = 0x00000000;
  // We reset the registers :
  m_pwRegister[r0] = 0x00000000;
  m_pwRegister[at] = 0x00000000;
  m_pwRegister[v0] = 0x00000000;
  m_pwRegister[v1] = 0x00000000;
  m_pwRegister[a0] = 0x00000000;
  m_pwRegister[a1] = 0x00000000;
  m_pwRegister[a2] = 0x00000000;
  m_pwRegister[a3] = 0x00000000;
  m_pwRegister[t0] = 0x00000000;
  m_pwRegister[t1] = 0x00000000;
  m_pwRegister[t2] = 0x00000000;
  m_pwRegister[t3] = 0x00000000;
  m_pwRegister[t4] = 0x00000000;
  m_pwRegister[t5] = 0x00000000;
  m_pwRegister[t6] = 0x00000000;
  m_pwRegister[t7] = 0x00000000;
  m_pwRegister[s0] = 0x00000000;
  m_pwRegister[s1] = 0x00000000;
  m_pwRegister[s2] = 0x00000000;
  m_pwRegister[s3] = 0x00000000;
  m_pwRegister[s4] = 0x00000000;
  m_pwRegister[s5] = 0x00000000;
  m_pwRegister[s6] = 0x00000000;
  m_pwRegister[s7] = 0x00000000;
  m_pwRegister[t8] = 0x00000000;
  m_pwRegister[t9] = 0x00000000;
  m_pwRegister[k0] = 0x00000000;
  m_pwRegister[k1] = 0x00000000;
  m_pwRegister[gp] = 0x10008000;
  m_pwRegister[sp] = 0x7FFFEFFC;
  m_pwRegister[s8] = 0x00000000;
  m_pwRegister[ra] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
RegisterBox::Display(void)
{
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      if(m_eIsActivated == true)
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)RegisterBox_enabled);
	}
      else
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)RegisterBox_disabled);
	}
      gdk_draw_drawable(pPixmap,
			pDrawingArea->style->fg_gc[GTK_WIDGET_STATE(pDrawingArea)],
			pPixmapTmp,0,0,
			m_pCoord[0].X,
			m_pCoord[0].Y,
			m_pCoord[1].X - m_pCoord[0].X,
			m_pCoord[1].Y - m_pCoord[0].Y);
      gdk_pixmap_unref(pPixmapTmp);
      gtk_widget_queue_draw_area (pDrawingArea,
				  0,0,
				  pDrawingArea->allocation.width,
				  pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:
      using namespace std;
      cout << hex << setfill('0') << internal << uppercase ;
      for(int r = 0 ; r < (MAX_REGISTERS/4) ; r++)
	{
	  cout << szRegister[r]    << " = 0x";
	  cout << setw(8) << m_pwRegister[r]    << "  ";
	  cout << szRegister[r+8]  << " = 0x";
	  cout << setw(8) << m_pwRegister[r+8]  << "  ";
	  cout << szRegister[r+16] << " = 0x";
	  cout << setw(8) << m_pwRegister[r+16] << "  ";
	  cout << szRegister[r+24] << " = 0x";
	  cout << setw(8) << m_pwRegister[r+24] << endl;
	}
      break;
    }
}

#endif // REGISTERBOX_CPP
