#ifndef ALU32_CPP
#define ALU32_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: ALU32
//
// BASE CLASSES: None.
//
// PURPOSE: A simple 32-bits ALU.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
//////////////////////////////////////////////////////////////////////////////

//Includes

#include "ALU32.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME:
//
// DESCRIPTION: This public method is used to get the Result output of the ALU.
//
// PARAMETERS:
//  - w1 : First operand.
//  - w2 : Second operand.
//  - aluOperation : The ALU operation to be performed. Legal operations are :
//           * AND  (0x00)
//           * OR   (0x01)
//           * ADD  (0x02)
//           * NOR  (0x04)
//           * XOR  (0x05)
//           * ADDU (0x06)
//           * SUB  (0x0A)
//           * SLT  (0x0B)
//           * SUBU (0x0E)
//           * SLTU (0x0F)
//
// RETURNS: The Result of the ALU operation.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
ALU32::Result(word w1,word w2,byte aluOperation)
{
  unsigned short b;
  word r=0x00000000;
  byte CarryIn,bNegate,aluOp;
  bNegate=Core::Bit(aluOperation,3);
  CarryIn=bNegate;
  aluOp=Core::Sel(aluOperation,2,0);
  for(b=0;b<32;b++)
    {
      r=r|(ALU1::Result(Core::Bit(w1,b),Core::Bit(w2,b),CarryIn,0,aluOp,bNegate)<<b);
      if(b!=31)
	CarryIn=ALU1::CarryOut(Core::Bit(w1,b),Core::Bit(w2,b),CarryIn,bNegate);
      else
	r=r|ALU1::Result(Core::Bit(w1,0),Core::Bit(w2,0),bNegate,ALU1::Set(Core::Bit(w1,b),Core::Bit(w2,b),CarryIn,aluOp,bNegate),aluOp,bNegate);
    }
  return r;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Zero
//
// DESCRIPTION: This public method is used to get the Zero output of the ALU.
//
// PARAMETERS:
//  - w1 : First operand.
//  - w2 : Second operand.
//  - aluOperation : The ALU operation to be performed. Legal operations are :
//           * AND  (0x00)
//           * OR   (0x01)
//           * ADD  (0x02)
//           * NOR  (0x04)
//           * XOR  (0x05)
//           * ADDU (0x06)
//           * SUB  (0x0A)
//           * SLT  (0x0B)
//           * SUBU (0x0E)
//           * SLTU (0x0F)
//
// RETURNS: The Zero output of the ALU.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
ALU32::Zero(word w1,word w2,byte aluOperation)
{
  return (ALU32::Result(w1,w2,aluOperation)==0x00000000)?0x01:0x00;
}

#endif // ALU32_CPP
