#ifndef MONOCYCLECONTROL_HPP
#define MONOCYCLECONTROL_HPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: MonocycleControl
//
// BASE CLASSES: None.
//
// PURPOSE: A monocycle datapath control unit implementation.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "../Core.hpp"

//Constants

#define MAX_MONOCYCLE_CONTROL_LINES 16

// This enum describes the monocycle control lines
enum { _RegDst_   = 0  , _Branch_   = 1  , _MemRead_  = 2  , _MemToReg_ = 3  ,
       _ALUOp_    = 4  , _MemWrite_ = 5  , _ReadSrc_  = 6  , _RegWrite_ = 7  ,
       _Jump_     = 8  , _JumpSrc_  = 9  , _Link_     = 10 , _Unsigned_ = 11 ,
       _DataSrc_  = 12 , _ZeroCmp_  = 13 , _ZeroInv_  = 14 , _ShiftOp_  = 15 };

// This array is used to match a number with the corresponding monocycle
// control line's name
extern const char* szMonocycleControlLine[MAX_MONOCYCLE_CONTROL_LINES];

//Types and classes

// This class describes the MIPS monocycle datapath control unit,
// using a Programmable Logic Array (PLA)
class MonocycleControl
{
public:
  static byte Result(int,word);
private:
  static byte RegWrite(word);
  static byte ReadSrc(word);
  static byte MemWrite(word);
  static byte ALUOp(word);
  static byte MemToReg(word);
  static byte MemRead(word);
  static byte Branch(word);
  static byte Jump(word);
  static byte RegDst(word);
  static byte Link(word);
  static byte ZeroInv(word);
  static byte ZeroCmp(word);
  static byte Unsigned(word);
  static byte DataSrc(word);
  static byte JumpSrc(word);
  static byte ShiftOp(word);
private:
  static byte is_sll(word);
  static byte is_srl(word);
  static byte is_sra(word);
  static byte is_sllv(word);
  static byte is_srlv(word);
  static byte is_srav(word);
  static byte is_jr(word);
  static byte is_jalr(word);
  static byte is_add(word);
  static byte is_addu(word);
  static byte is_sub(word);
  static byte is_subu(word);
  static byte is_and(word);
  static byte is_or(word);
  static byte is_xor(word);
  static byte is_nor(word);
  static byte is_slt(word);
  static byte is_sltu(word);
  static byte is_bltz(word);
  static byte is_bgez(word);
  static byte is_bltzal(word);
  static byte is_bgezal(word);
  static byte is_j(word);
  static byte is_jal(word);
  static byte is_beq(word);
  static byte is_bne(word);
  static byte is_blez(word);
  static byte is_bgtz(word);
  static byte is_addi(word);
  static byte is_addiu(word);
  static byte is_slti(word);
  static byte is_sltiu(word);
  static byte is_andi(word);
  static byte is_ori(word);
  static byte is_xori(word);
  static byte is_lw(word);
  static byte is_sw(word);
};

#endif // MONOCYCLECONTROL_HPP
