%{

#include <iostream>
#include <iomanip>
#include <cstdlib>

#include "../../../Core/Core.hpp"

#define YYSTYPE word

#include "Mips.tab.h"

unsigned int nLine = 1;
unsigned int yyerrcount = 0;

using namespace std;

%}

REGEXP_SPACE              [ \t]+
REGEXP_COMMA              [ \t]*[,][ \t]*
REGEXP_DIGIT              [0-9]
REGEXP_LBRACKET           [(]
REGEXP_RBRACKET           [)]
REGEXP_ADDRESS            "0x"([0-9a-fA-F]{1,8})
REGEXP_IMMEDIATE          [-]?{REGEXP_DIGIT}+
REGEXP_NEWLINE            [\r\n]+
REGEXP_COMMENT            {REGEXP_SPACE}*[;#].*
REGEXP_ERROR              [^ \t\r\n,();#]+

REGEXP_REGISTER_NUMERIC   [$](([12]*[0-9])|(3[01]))
REGEXP_REGISTER_ZERO      "$zero"|"$r0"
REGEXP_REGISTER_AT        "$at"
REGEXP_REGISTER_V0        "$v0"
REGEXP_REGISTER_V1        "$v1"
REGEXP_REGISTER_A0        "$a0"
REGEXP_REGISTER_A1        "$a1"
REGEXP_REGISTER_A2        "$a2"
REGEXP_REGISTER_A3        "$a3"
REGEXP_REGISTER_T0        "$t0"
REGEXP_REGISTER_T1        "$t1"
REGEXP_REGISTER_T2        "$t2"
REGEXP_REGISTER_T3        "$t3"
REGEXP_REGISTER_T4        "$t4"
REGEXP_REGISTER_T5        "$t5"
REGEXP_REGISTER_T6        "$t6"
REGEXP_REGISTER_T7        "$t7"
REGEXP_REGISTER_S0        "$s0"
REGEXP_REGISTER_S1        "$s1"
REGEXP_REGISTER_S2        "$s2"
REGEXP_REGISTER_S3        "$s3"
REGEXP_REGISTER_S4        "$s4"
REGEXP_REGISTER_S5        "$s5"
REGEXP_REGISTER_S6        "$s6"
REGEXP_REGISTER_S7        "$s7"
REGEXP_REGISTER_T8        "$t8"
REGEXP_REGISTER_T9        "$t9"
REGEXP_REGISTER_K0        "$k0"
REGEXP_REGISTER_K1        "$k1"
REGEXP_REGISTER_GP        "$gp"
REGEXP_REGISTER_SP        "$sp"
REGEXP_REGISTER_FP        "$fp"
REGEXP_REGISTER_RA        "$ra"
                          
REGEXP_OPCODE_NOP         "nop"
REGEXP_OPCODE_J           "j"
REGEXP_OPCODE_JAL         "jal"
REGEXP_OPCODE_ADD         "add"
REGEXP_OPCODE_ADDU        "addu"
REGEXP_OPCODE_AND         "and"
REGEXP_OPCODE_NOR         "nor"
REGEXP_OPCODE_OR          "or"
REGEXP_OPCODE_SLT         "slt"
REGEXP_OPCODE_SLTU        "sltu"
REGEXP_OPCODE_SUB         "sub"
REGEXP_OPCODE_SUBU        "subu"
REGEXP_OPCODE_XOR         "xor"
REGEXP_OPCODE_SLLV        "sllv"
REGEXP_OPCODE_SRAV        "srav"
REGEXP_OPCODE_SRLV        "srlv"
REGEXP_OPCODE_SLL         "sll"
REGEXP_OPCODE_SRA         "sra"
REGEXP_OPCODE_SRL         "srl"
REGEXP_OPCODE_BEQ         "beq"
REGEXP_OPCODE_BNE         "bne"
REGEXP_OPCODE_BGEZ        "bgez"
REGEXP_OPCODE_BGEZAL      "bgezal"
REGEXP_OPCODE_BGTZ        "bgtz"
REGEXP_OPCODE_BLEZ        "blez"
REGEXP_OPCODE_BLTZAL      "bltzal"
REGEXP_OPCODE_BLTZ        "bltz"
REGEXP_OPCODE_JR          "jr"
REGEXP_OPCODE_JALR        "jalr"
REGEXP_OPCODE_LW          "lw"
REGEXP_OPCODE_SW          "sw"
REGEXP_OPCODE_ADDI        "addi"
REGEXP_OPCODE_ADDIU       "addiu"
REGEXP_OPCODE_ANDI        "andi"
REGEXP_OPCODE_ORI         "ori"
REGEXP_OPCODE_XORI        "xori"
REGEXP_OPCODE_SLTI        "slti"
REGEXP_OPCODE_SLTIU       "sltiu"

REGEXP_ASMDIR_ALIGN       ".align ".*
REGEXP_ASMDIR_ASCII       ".ascii ".*
REGEXP_ASMDIR_ASCIIZ      ".asciiz ".*
REGEXP_ASMDIR_BYTE        ".byte ".*
REGEXP_ASMDIR_DATA        ".data".*
REGEXP_ASMDIR_DOUBLE      ".double ".*
REGEXP_ASMDIR_EXTERN      ".extern ".*
REGEXP_ASMDIR_FLOAT       ".float ".*
REGEXP_ASMDIR_GLOBL       ".globl ".*
REGEXP_ASMDIR_HALF        ".half ".*
REGEXP_ASMDIR_KDATA       ".kdata".*
REGEXP_ASMDIR_KTEXT       ".ktext".*
REGEXP_ASMDIR_SPACE       ".space ".*
REGEXP_ASMDIR_TEXT        ".text".*
REGEXP_ASMDIR_WORD        ".word ".*

%%

{REGEXP_OPCODE_NOP}                     { yylval = 0x00000000; return OPCODE_NOARG;    }
{REGEXP_OPCODE_J}{REGEXP_SPACE}         { yylval = 0x08000000; return OPCODE_LABEL;    }
{REGEXP_OPCODE_JAL}{REGEXP_SPACE}       { yylval = 0x0C000000; return OPCODE_LABEL;    }
{REGEXP_OPCODE_ADD}{REGEXP_SPACE}       { yylval = 0x00000020; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_ADDU}{REGEXP_SPACE}      { yylval = 0x00000021; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_AND}{REGEXP_SPACE}       { yylval = 0x00000024; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_NOR}{REGEXP_SPACE}       { yylval = 0x00000027; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_OR}{REGEXP_SPACE}        { yylval = 0x00000025; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_SLT}{REGEXP_SPACE}       { yylval = 0x0000002A; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_SLTU}{REGEXP_SPACE}      { yylval = 0x0000002B; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_SUB}{REGEXP_SPACE}       { yylval = 0x00000022; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_SUBU}{REGEXP_SPACE}      { yylval = 0x00000023; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_XOR}{REGEXP_SPACE}       { yylval = 0x00000026; return OPCODE_RD_RS_RT; }
{REGEXP_OPCODE_SLLV}{REGEXP_SPACE}      { yylval = 0x00000004; return OPCODE_RD_RT_RS; }
{REGEXP_OPCODE_SRAV}{REGEXP_SPACE}      { yylval = 0x00000007; return OPCODE_RD_RT_RS; }
{REGEXP_OPCODE_SRLV}{REGEXP_SPACE}      { yylval = 0x00000006; return OPCODE_RD_RT_RS; }
{REGEXP_OPCODE_SLL}{REGEXP_SPACE}       { yylval = 0x00000000; return OPCODE_RD_RT_SH; }
{REGEXP_OPCODE_SRA}{REGEXP_SPACE}       { yylval = 0x00000003; return OPCODE_RD_RT_SH; }
{REGEXP_OPCODE_SRL}{REGEXP_SPACE}       { yylval = 0x00000002; return OPCODE_RD_RT_SH; }
{REGEXP_OPCODE_BEQ}{REGEXP_SPACE}       { yylval = 0x10000000; return OPCODE_RS_RT_CT; }
{REGEXP_OPCODE_BNE}{REGEXP_SPACE}       { yylval = 0x14000000; return OPCODE_RS_RT_CT; }
{REGEXP_OPCODE_BGEZ}{REGEXP_SPACE}      { yylval = 0x04010000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_BGEZAL}{REGEXP_SPACE}    { yylval = 0x04110000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_BGTZ}{REGEXP_SPACE}      { yylval = 0x1C000000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_BLEZ}{REGEXP_SPACE}      { yylval = 0x18000000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_BLTZAL}{REGEXP_SPACE}    { yylval = 0x04100000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_BLTZ}{REGEXP_SPACE}      { yylval = 0x04000000; return OPCODE_RS_CT;    }
{REGEXP_OPCODE_JR}{REGEXP_SPACE}        { yylval = 0x00000008; return OPCODE_RS;       }
{REGEXP_OPCODE_JALR}{REGEXP_SPACE}      { yylval = 0x00000009; return OPCODE_RS;       }
{REGEXP_OPCODE_LW}{REGEXP_SPACE}        { yylval = 0x8C000000; return OPCODE_RT_CT_RS; }
{REGEXP_OPCODE_SW}{REGEXP_SPACE}        { yylval = 0xAC000000; return OPCODE_RT_CT_RS; }
{REGEXP_OPCODE_ADDI}{REGEXP_SPACE}      { yylval = 0x20000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_ADDIU}{REGEXP_SPACE}     { yylval = 0x24000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_ANDI}{REGEXP_SPACE}      { yylval = 0x30000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_ORI}{REGEXP_SPACE}       { yylval = 0x34000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_XORI}{REGEXP_SPACE}      { yylval = 0x38000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_SLTI}{REGEXP_SPACE}      { yylval = 0x28000000; return OPCODE_RT_RS_CT; }
{REGEXP_OPCODE_SLTIU}{REGEXP_SPACE}     { yylval = 0x2C000000; return OPCODE_RT_RS_CT; }

{REGEXP_REGISTER_NUMERIC} { yylval = strtoul(yytext+1,NULL,10); return REG; }
{REGEXP_REGISTER_ZERO}    { yylval = 0;  return REG; }
{REGEXP_REGISTER_AT}      { yylval = 1;  return REG; }
{REGEXP_REGISTER_V0}      { yylval = 2;  return REG; }
{REGEXP_REGISTER_V1}      { yylval = 3;  return REG; }
{REGEXP_REGISTER_A0}      { yylval = 4;  return REG; }
{REGEXP_REGISTER_A1}      { yylval = 5;  return REG; }
{REGEXP_REGISTER_A2}      { yylval = 6;  return REG; }
{REGEXP_REGISTER_A3}      { yylval = 7;  return REG; }
{REGEXP_REGISTER_T0}      { yylval = 8;  return REG; }
{REGEXP_REGISTER_T1}      { yylval = 9;  return REG; }
{REGEXP_REGISTER_T2}      { yylval = 10; return REG; }
{REGEXP_REGISTER_T3}      { yylval = 11; return REG; }
{REGEXP_REGISTER_T4}      { yylval = 12; return REG; }
{REGEXP_REGISTER_T5}      { yylval = 13; return REG; }
{REGEXP_REGISTER_T6}      { yylval = 14; return REG; }
{REGEXP_REGISTER_T7}      { yylval = 15; return REG; }
{REGEXP_REGISTER_S0}      { yylval = 16; return REG; }
{REGEXP_REGISTER_S1}      { yylval = 17; return REG; }
{REGEXP_REGISTER_S2}      { yylval = 18; return REG; }
{REGEXP_REGISTER_S3}      { yylval = 19; return REG; }
{REGEXP_REGISTER_S4}      { yylval = 20; return REG; }
{REGEXP_REGISTER_S5}      { yylval = 21; return REG; }
{REGEXP_REGISTER_S6}      { yylval = 22; return REG; }
{REGEXP_REGISTER_S7}      { yylval = 23; return REG; }
{REGEXP_REGISTER_T8}      { yylval = 24; return REG; }
{REGEXP_REGISTER_T9}      { yylval = 25; return REG; }
{REGEXP_REGISTER_K0}      { yylval = 26; return REG; }
{REGEXP_REGISTER_K1}      { yylval = 27; return REG; }
{REGEXP_REGISTER_GP}      { yylval = 28; return REG; }
{REGEXP_REGISTER_SP}      { yylval = 29; return REG; }
{REGEXP_REGISTER_FP}      { yylval = 30; return REG; }
{REGEXP_REGISTER_RA}      { yylval = 31; return REG; }

{REGEXP_ASMDIR_ALIGN}     {  }
{REGEXP_ASMDIR_ASCII}     {  }
aaa{REGEXP_ASMDIR_ASCIIZ}    {  }
{REGEXP_ASMDIR_BYTE}      {  }
{REGEXP_ASMDIR_DATA}      {  }
{REGEXP_ASMDIR_DOUBLE}    {  }
{REGEXP_ASMDIR_EXTERN}    {  }
{REGEXP_ASMDIR_FLOAT}     {  }
{REGEXP_ASMDIR_GLOBL}     {  }
{REGEXP_ASMDIR_HALF}      {  }
{REGEXP_ASMDIR_KDATA}     {  }
{REGEXP_ASMDIR_KTEXT}     {  }
{REGEXP_ASMDIR_SPACE}     {  }
{REGEXP_ASMDIR_TEXT}      {  }
{REGEXP_ASMDIR_WORD}      {  }

{REGEXP_COMMA}            { return COMMA;               }
{REGEXP_SPACE}            {                             }
{REGEXP_COMMENT}          {                             }
{REGEXP_NEWLINE}          { nLine++; return NEWLINE;    }
{REGEXP_LBRACKET}         { return LBRACKET;            }
{REGEXP_RBRACKET}         { return RBRACKET;            }
{REGEXP_IMMEDIATE}        { yylval = strtoul(yytext,NULL,10); return IMM;     }
{REGEXP_ADDRESS}          { yylval = strtoul(yytext,NULL,16); return ADDRESS; }
{REGEXP_ERROR}            { cerr << "Line " << setw(3) << nLine << " :: "
                                 << "unknown token \'" << yytext << "\'" << endl;
                            yyerrcount++; return ERROR; }

%%
