%{

#include <iostream>
#include <iomanip>

#include "../../../Core/Core.hpp"
#include "../../../Core/Memory.hpp"

#define YYSTYPE word
  
  extern YYSTYPE yylval;
  extern char* yytext;
  extern unsigned int yyerrcount;
  extern int yylex();
  extern Memory yyInstrMem;
  extern unsigned int nLine;
  
  using namespace std;
  
int yyerror(char* szMsg)
{
  cerr << "Line " << setw(3) << nLine << " :: " << szMsg << " near \'" << yytext << "\'" << endl;
  yyerrcount++;
  return 0;
}

%}

%token NEWLINE
%token COMMA
%token IMM
%token REG
%token LBRACKET
%token RBRACKET
%token ADDRESS

%token OPCODE_NOARG
%token OPCODE_LABEL
%token OPCODE_RD_RS_RT
%token OPCODE_RD_RT_RS
%token OPCODE_RD_RT_SH
%token OPCODE_RD_RS_CT
%token OPCODE_RS_RT_CT
%token OPCODE_RS_CT
%token OPCODE_RS
%token OPCODE_RT_CT_RS
%token OPCODE_RT_RS_CT

%token ERROR

%start INPUT
%%

INPUT:
   /* Empty */
 | INPUT INSTRUCTION NEWLINE { yyInstrMem[END_ADDR] = $2;
                               END_ADDR += 0x00000004; }
 | INPUT NEWLINE { }
 | INPUT error { }
 ;

INSTRUCTION:
   OPCODE_NOARG                                                 { $$ = $1; }
 | OPCODE_LABEL       ADDRESS                                   { $$ = $1 | Core::Sel($2>>2,25,0); }
 | OPCODE_RD_RS_RT    REG  COMMA  REG  COMMA  REG               { $$ = $1 | ($2<<11) | ($4<<21) | ($6<<16); }
 | OPCODE_RD_RT_RS    REG  COMMA  REG  COMMA  REG               { $$ = $1 | ($2<<11) | ($4<<16) | ($6<<21); }
 | OPCODE_RD_RT_SH    REG  COMMA  REG  COMMA  IMM               { $$ = $1 | ($2<<11) | ($4<<16) | (Core::Sel($6,4,0)<<6); }
 | OPCODE_RD_RS_CT    REG  COMMA  REG  COMMA  IMM               { $$ = $1 | ($2<<11) | ($4<<21) | Core::Sel($6,15,0); }
 | OPCODE_RS_RT_CT    REG  COMMA  REG  COMMA  IMM               { $$ = $1 | ($2<<21) | ($4<<16) | Core::Sel($6>>2,15,0); }
 | OPCODE_RS_CT       REG  COMMA  IMM                           { $$ = $1 | ($2<<21) | Core::Sel($4>>2,15,0); }
 | OPCODE_RS          REG                                       { $$ = $1 | ($2<<21) | (31<<11); }
 | OPCODE_RT_CT_RS    REG  COMMA  IMM  LBRACKET  REG  RBRACKET  { $$ = $1 | ($2<<16) | Core::Sel($4,15,0) | ($6<<21); }
 | OPCODE_RT_RS_CT    REG  COMMA  REG  COMMA  IMM               { $$ = $1 | ($2<<16) | ($4<<21) | Core::Sel($6,15,0); }
 ;

%%

int parse(void)
{
  yyerrcount = 0;
  START_ADDR = 0x00400000;
  END_ADDR = START_ADDR;
  return yyparse();
}
