#ifndef INTERFACE_CPP
#define INTERFACE_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Interface
//
// BASE CLASSES: N/A.
//
// PURPOSE: Graphical User Interface.
//
// AUTHOR: Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Interface.hpp"
#include "DataPaths/Monocycle/MonocycleDataPath.hpp"

//Constants

GtkWidget* pWindow = NULL;
GtkWidget* pTable = NULL;
GtkWidget* pButton[5];
GtkWidget* pLabel = NULL;
GtkWidget* pMenuBar = NULL;
GtkItemFactory* pItemFactory = NULL;
GtkAccelGroup* pAccel = NULL;
GtkWidget* pDrawingArea = NULL;
GdkPixmap* pPixmap = NULL;
GdkPixmap* pPixmapTmp = NULL;
GdkColor Color;
GdkColormap* ColorMap;
GdkGC* gc;
gchar* sUtf8;

/* Menu Items */
GtkItemFactoryEntry MenuItem[MAX_MENU_ITEMS] = {
    { "/_File",NULL,NULL,0,"<Branch>" },
    { "/File/_Load","<ctrl>L",(void (*)())OnLoad,0,"<StockItem>",GTK_STOCK_OPEN },
    { "/File/_Next Cycle","<ctrl>N",(void (*)())OnNext,0,"<StockItem>",GTK_STOCK_EXECUTE },
    { "/File/_Reset","<ctrl>R",(void (*)())OnReset,0,"<StockItem>",GTK_STOCK_CLEAR },
    { "/File/Sep1",NULL,NULL,0,"<Separator>" },
    { "/File/_Quit",NULL,(void (*)())OnQuit,0,"<StockItem>",GTK_STOCK_QUIT},
    { "/_?",NULL,NULL,0,"<Branch>" },
    { "/?/_About","<ctrl>A",(void (*)())OnAbout,0,"<StockItem>",GTK_STOCK_HELP}
};

//Externals

extern FILE* yyin;
extern int parse(void);
extern unsigned int yyerrcount;
extern DataPath* DP;
extern Memory yyInstrMem;
extern word wNewPC;
extern word wOldPC;

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnLoad
//
// DESCRIPTION: This function is called when the 'Load File' button is pressed.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnLoad(GtkWidget* pWidget)
{
    GtkWidget* pFileSelection;
    const gchar* szPath;

    /* We create the window */
    pFileSelection = gtk_file_selection_new("The Koyote Project");

    /* Set window type to MODAL */
    gtk_window_set_modal(GTK_WINDOW(pFileSelection),TRUE);

    /* Display the window */
    switch(gtk_dialog_run(GTK_DIALOG(pFileSelection)))
      {
      case GTK_RESPONSE_OK:
	szPath =
	  gtk_file_selection_get_filename(GTK_FILE_SELECTION(pFileSelection));
	yyin = fopen(szPath,"r");
	if(yyin == (FILE*)0)
	  {
	    gtk_label_set_text(GTK_LABEL(pLabel),"Can\'t open file !");
	  }
	else
	  {
	    OnReset(pWidget);
	    parse();
	    fclose(yyin);
	    AllocTable* pInstrTable;
	    pInstrTable = yyInstrMem.GetAllocTable();
	    ((MonocycleDataPath*)DP)->Init(pInstrTable,(AllocTable*)0);
	    if(yyerrcount == 0)
	      {
		gtk_label_set_text(GTK_LABEL(pLabel),
				   "File loaded. Click the \'Next Cycle\'"
				   " button to start.");
	      }
	    else
	      {
		  gtk_label_set_text(GTK_LABEL(pLabel),
				     "Your file contains errors. Check the "
				     "console output to see a detailed error "
				     "report.");
	      }
	  }
	break;
      default:
	break;
      }
    gtk_widget_destroy(pFileSelection);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnNext
//
// DESCRIPTION: This function is called when the 'Next Cycle' button is pressed
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnNext(GtkWidget* pWidget)
{
  if(START_ADDR < END_ADDR) // Memory not empty
    {
      if((wNewPC >= START_ADDR) && (wNewPC < END_ADDR))
	{
	  wOldPC = wNewPC;
	  wNewPC = DP->Cycle();
	  gtk_label_set_text(GTK_LABEL(pLabel),"");
	}
      else
	{
	  if((wOldPC + 0x00000004) < END_ADDR) // Virtual SIGSEGV !
	    {
	      OnReset(pWidget);
	      gtk_label_set_text(GTK_LABEL(pLabel),
				 "Bad Address. "
				 "DataPath Reset. "
				 "Please load a MIPS assembly file...");
	    }
	  else
	    {
	      gtk_label_set_text(GTK_LABEL(pLabel),
				 "No more instruction to display. "
				 "Please load a MIPS assembly file...");
	    }
	}
    }
  else // Memory empty
    {
      gtk_label_set_text(GTK_LABEL(pLabel),"Instruction memory is empty ! "
			 "Please load a MIPS assembly file...");
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnReset
//
// DESCRIPTION: This function is called when the 'Reset' button is pressed.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnReset(GtkWidget* pWidget)
{
  delete DP; // Most effective...
  yyInstrMem.Reset();
  wOldPC = START_ADDR;
  wNewPC = START_ADDR;
  END_ADDR = START_ADDR;
  DP = new MonocycleDataPath(DISPLAY_GRAPHIC);
  DP->Display();
  gtk_label_set_text(GTK_LABEL(pLabel),"DataPath Reset. "
		     "Please load a MIPS assembly file...");
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnError
//
// DESCRIPTION: This function is called when the '!' button is pressed.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnError(GtkWidget* pWidget)
{
  // gtk_label_set_text(GTK_LABEL(pLabel),"");
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnQuit
//
// DESCRIPTION: This function is called when the 'Quit' button is pressed.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnQuit(GtkWidget* pWidget)
{
    GtkWidget *pQuestion;

    pQuestion = gtk_message_dialog_new(GTK_WINDOW(pWindow),
        GTK_DIALOG_MODAL,
        GTK_MESSAGE_QUESTION,
        GTK_BUTTONS_YES_NO,
        "Do  you  really  want  to\n"
        "quit The Koyote Project ?");

    switch(gtk_dialog_run(GTK_DIALOG(pQuestion)))
    {
    case GTK_RESPONSE_YES:
      gtk_main_quit();
      delete DP;
      yyInstrMem.Reset();
      break;
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_NO:
      gtk_widget_destroy(pQuestion);
      break;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnDestroy
//
// DESCRIPTION: This function is called when the 'Destroy' button is pressed.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnDestroy(GtkWidget* pWidget)
{
  gtk_main_quit();
  delete DP;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnAbout
//
// DESCRIPTION: This function displays the 'About' window.
//
// PARAMETERS: A GTK Widget.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void OnAbout(GtkWidget* pWidget)
{
    GtkWidget* pAbout;

    pAbout = gtk_message_dialog_new(GTK_WINDOW(pWindow),
				    GTK_DIALOG_MODAL,
				    GTK_MESSAGE_INFO,
				    GTK_BUTTONS_OK,
				    "The Koyote Project Version "
				    KOYOTE_VERSION
				    "\nhttp://koyoteproject.free.fr\n\n"
				    "DONVAL Brice & MOREAU Nicolas\n"
				    "2003-2004 @ IUT Fontainebleau");

    gtk_dialog_run(GTK_DIALOG(pAbout));
    gtk_widget_destroy(pAbout);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ExposeEvent
//
// DESCRIPTION: Fills the screen with the background pixmap.
//
// PARAMETERS: GTK Widget, GDK EventExpose
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint ExposeEvent(GtkWidget* pWidget,GdkEventExpose* pEvent)
{
  gdk_draw_pixmap(pWidget->window,
                  pWidget->style->fg_gc[GTK_WIDGET_STATE(pWidget)],
                  pPixmap,
                  pEvent->area.x, pEvent->area.y,
                  pEvent->area.x, pEvent->area.y,
                  pEvent->area.width, pEvent->area.height);

  return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ConfigureEvent
//
// DESCRIPTION: Creates a new background pixmap.
//
// PARAMETERS: GTK Widget, GDK EventConfigure.
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint ConfigureEvent(GtkWidget* pWidget,GdkEventConfigure* pEvent)
{
  if (pPixmap != NULL)
      gdk_pixmap_unref(pPixmap);

  pPixmap = gdk_pixmap_new(pWidget->window,
			   pWidget->allocation.width,
			   pWidget->allocation.height,
			   -1);

  /* White background */
  gdk_draw_rectangle(pPixmap,
		     pWidget->style->white_gc,
		     TRUE,
		     0, 0,
		     pWidget->allocation.width,
		     pWidget->allocation.height);

  ////**////

  return TRUE;
}

#endif // INTERFACE_CPP
