#ifndef KOYOTE_CPP
#define KOYOTE_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: koyote
//
// BASE CLASSES: N/A.
//
// PURPOSE: Main program module.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "koyote.hpp"

//Constants

//Externals

extern int parse(void);
extern unsigned int yyerrcount;
extern FILE* yyin;

Memory yyInstrMem;
DataPath* DP;
word wNewPC;
word wOldPC;

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: usage
//
// DESCRIPTION: This function prints usage message.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
usage(void)
{
  using namespace std;
  cerr << "Usage: koyote [-c|--console|-g|--graphic|-h|--help|-v|--version]"
       << endl;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: help
//
// DESCRIPTION: This function prints help information.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
help(void)
{
  using namespace std;
  usage();
  cout << endl << "Operation modes:" << endl
       << "  -c, --console:        Text mode" << endl
       << "  -g, --graphic:        Graphic mode (Default mode)" << endl
       << "  -v, --version:        Version info" << endl
       << "  -h, --help:           Display this help" << endl << endl;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: version
//
// DESCRIPTION: This function prints version information.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
version(void)
{
  using namespace std;
  cout << "The KOYOTE Project - KOYOTE Version " << KOYOTE_VERSION
       << "." << endl << endl
       << "Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL."
       << endl << "Visit http://koyoteproject.free.fr for updates."
       << endl << endl
       << "This is FREE SOFTWARE; see the source for copying"
       << " conditions.  There is NO" << endl << "WARRANTY;"
       << " not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR"
       << " PURPOSE." << endl << endl;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: console
//
// DESCRIPTION: This function implements the "console" (text-based) mode.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
console(void)
{
  using namespace std;
  
  char filename[256] = {0};
  cout << "Welcome to The KOYOTE Project !... "
       << "Please load a MIPS assembly file..." << endl << endl << "File: ";
  cin >> setw(255) >> filename;
  
  yyin = fopen(filename,"r");
  if(yyin == (FILE*)0)
    {
      cerr << endl << "koyote: *** [" << filename << "] Can\'t open file ! ***"
	   << endl << endl;
    }
  else
    {
      parse();
      fclose(yyin);
      
      if(yyerrcount > 0)
	{
	  cerr << endl << "koyote: *** [" << filename << "] "
	       << yyerrcount << " error";
	  if(yyerrcount > 1) cerr << "s";
	  cerr << " ***" << endl << "        *** Warning: Instruction Memory"
	       << " initialization incomplete ! ***" << endl;
	}
      if(START_ADDR != END_ADDR) // Memory initialized
	{
	  DP = new MonocycleDataPath(DISPLAY_TEXT);
	  AllocTable* pInstrTable = yyInstrMem.GetAllocTable();
	  ((MonocycleDataPath*)DP)->Init(pInstrTable,(AllocTable*)0);
	  
	  wNewPC = START_ADDR;
	  wOldPC = START_ADDR;
	  while((wNewPC >= START_ADDR) && (wNewPC < END_ADDR))
	    {
	      getc(stdin);
	      wOldPC = wNewPC;
	      wNewPC = DP->Cycle();
	    }
	  if((wOldPC + 0x00000004) < END_ADDR) // Virtual SIGSEGV !
	    cerr << "koyote: *** [" << filename << "] Segmentation Fault ! ***"
		 << endl << "        BAD ADDRESS: 0x" << hex << uppercase
		 << setfill('0') << setw(8) << wNewPC << endl;
	  else
	    cout << "koyote: "
		 << "No more instruction to display." << endl;
	  cout << endl;
	  
	  delete DP;
	  yyInstrMem.Reset();
	}
      else // Memory empty !
	cerr << endl << "koyote: *** [" << filename << "] Instruction Memory"
	     << " empty ! ***" << endl << endl;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: graphic
//
// DESCRIPTION: This function implements the "graphic" (GTK+) mode.
//
// PARAMETERS: Same as 'main'.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
graphic(int nArgc,char* pszArgv[])
{
  START_ADDR = 0x00400000;
  END_ADDR = START_ADDR;
  wOldPC = START_ADDR;
  wNewPC = START_ADDR;
  DP = new MonocycleDataPath(DISPLAY_GRAPHIC);
  
  gtk_init(&nArgc,&pszArgv);
  
  /* Closing Frame */
  pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(pWindow),"The Koyote Project");
  gtk_widget_set_usize(pWindow,1000,700);
  gtk_window_set_resizable(GTK_WINDOW(pWindow),FALSE);
  g_signal_connect(G_OBJECT(pWindow),"destroy",
		   G_CALLBACK(/*gtk_main_quit*/OnDestroy),NULL);
  
  pTable=gtk_table_new(500,500,TRUE);
  gtk_container_add(GTK_CONTAINER(pWindow),GTK_WIDGET(pTable));
  
  /* Accelerator table creation */
  pAccel = gtk_accel_group_new();
  
  /* Menu creation */
  pItemFactory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",pAccel);
  /* Get menu items */
  gtk_item_factory_create_items(pItemFactory,MAX_MENU_ITEMS,
				MenuItem,(GtkWidget*)pWindow);
  /* Get widget to display menu */
  pMenuBar = gtk_item_factory_get_widget(pItemFactory,"<main>");
  /* Attach the menu */
  gtk_table_attach_defaults(GTK_TABLE(pTable),pMenuBar,0,500,0,25);
  
  /* Attach accelerators to the window */
  gtk_window_add_accel_group(GTK_WINDOW(pWindow),pAccel);
  
  /* Buttons */
  pButton[0]= gtk_button_new_with_label("Load File");
  pButton[1]= gtk_button_new_with_label("Next Cycle");
  pButton[2]= gtk_button_new_with_label("Reset");
  pButton[3]= gtk_button_new_with_label("!");
  pButton[4]= gtk_button_new_with_label("Quit");
  
  /* Connect signal handlers with corresponding buttons */
  g_signal_connect(G_OBJECT(pButton[0]),"clicked",
		   G_CALLBACK(OnLoad),NULL);
  g_signal_connect(G_OBJECT(pButton[1]),"clicked",
		   G_CALLBACK(OnNext),NULL);
  g_signal_connect(G_OBJECT(pButton[2]),"clicked",
		   G_CALLBACK(OnReset),NULL);
  g_signal_connect(G_OBJECT(pButton[3]),"clicked",
		   G_CALLBACK(OnError),NULL);
  g_signal_connect(G_OBJECT(pButton[4]),"clicked",
		   G_CALLBACK(OnQuit),NULL);
  
  /* Attach the buttons to the window */
  gtk_table_attach_defaults(GTK_TABLE(pTable),pButton[0],  5, 55,485,499);
  gtk_table_attach_defaults(GTK_TABLE(pTable),pButton[1], 60,110,485,499);
  gtk_table_attach_defaults(GTK_TABLE(pTable),pButton[2],115,165,485,499);
  gtk_table_attach_defaults(GTK_TABLE(pTable),pButton[3],425,440,485,499);
  gtk_table_attach_defaults(GTK_TABLE(pTable),pButton[4],445,495,485,499);
      
  pLabel = gtk_label_new("Welcome to The Koyote Project !...   "
			 "  Please load a MIPS assembly file...");
  
  gtk_table_attach_defaults(GTK_TABLE(pTable),pLabel,170,425,485,499);
  
  /* Drawing Area creation */
  pDrawingArea = gtk_drawing_area_new();
  gtk_table_attach_defaults(GTK_TABLE(pTable),pDrawingArea,0,500,25,482);
  
  /* Connect the Drawing Area signal handlers */
  gtk_signal_connect(GTK_OBJECT (pDrawingArea),
		     "expose_event",
		     (GtkSignalFunc)ExposeEvent,NULL);
  gtk_signal_connect(GTK_OBJECT(pDrawingArea),
		     "configure_event",
		     (GtkSignalFunc)ConfigureEvent,NULL);
  //     gtk_signal_connect(GTK_OBJECT (pDrawingArea),"motion_notify_event",(GtkSignalFunc)MotionNotifyEvent,NULL);
  //     gtk_signal_connect(GTK_OBJECT (pDrawingArea),"button_press_event",(GtkSignalFunc)ButtonPressEvent,NULL);
  
  gtk_widget_set_events(pDrawingArea,GDK_EXPOSURE_MASK
			| GDK_LEAVE_NOTIFY_MASK
			| GDK_BUTTON_PRESS_MASK
			| GDK_POINTER_MOTION_MASK
			| GDK_POINTER_MOTION_HINT_MASK);
  
  gtk_widget_show_all(pWindow);

  DP->Display();      
  gtk_main();  
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: main
//
// DESCRIPTION: main function.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
int
main(int nArgc,char* pszArgv[])
{
  switch(nArgc)
    {
    case 2:
      if((strcmp(pszArgv[1],"--console") == 0) ||
	 (strcmp(pszArgv[1],"-c") == 0))
	console();
      else if((strcmp(pszArgv[1],"--graphic") == 0) ||
	      (strcmp(pszArgv[1],"-g") == 0))
	graphic(nArgc,pszArgv);
      else if((strcmp(pszArgv[1],"--version") == 0) ||
	      (strcmp(pszArgv[1],"-v") == 0))
	version();
      else if((strcmp(pszArgv[1],"--help") == 0) ||
	      (strcmp(pszArgv[1],"-h") == 0))
	help();
      else
	usage();
      break;
    case 1:
      graphic(nArgc,pszArgv);
      break;
    default:
      usage();
    }
  return 0;
}

#endif // KOYOTE_CPP
